import typing as t

import pydantic

from taktile_auth.entities.resource import Resource, ResourceDefinition
from taktile_auth.enums import Action


class Permission(pydantic.BaseModel):
    actions: t.Set[Action]
    resource: Resource
    resource_name: str

    def __contains__(self, query: "Permission") -> bool:
        if not query.actions.issubset(self.actions):
            return False
        if query.resource_name != self.resource_name:
            return False
        return query.resource in self.resource

    def __repr__(self) -> str:
        actions = "+".join(sorted(self.actions))
        vals = ",".join(self.resource.dict().values())
        return f"{actions}:{self.resource_name}/{vals}"

    def __hash__(self) -> int:
        return hash(
            (
                tuple(sorted(list(self.actions))),
                self.resource_name,
                self.resource,
            )
        )


Permission.update_forward_refs()


class PermissionDefinition(pydantic.BaseModel):
    resource_definition: ResourceDefinition

    def build(self, *, actions: t.Set[Action], **kwargs: str) -> Permission:
        extra_args = {
            arg: "*"
            for arg in set(self.resource_definition.args.keys()).difference(
                kwargs
            )
        }

        return Permission(
            actions=actions,
            resource=self.resource_definition.get_resource()(
                **kwargs, **extra_args
            ),
            resource_name=self.resource_definition.resource_name,
        )



import typing as t


def parse_body(clauses: t.List[str]) -> t.List[t.Dict[str, t.Any]]:
    parsed_clauses = []
    for clause in clauses:
        if _is_permission(clause):
            parsed_clauses.append(parse_permission(clause))
        else:
            parsed_clauses.append(parse_sub_role(clause))
    return parsed_clauses


def _is_permission(clause: str) -> bool:
    return ":" in clause


def parse_permission(clause: str) -> t.Dict[str, t.Any]:
    action_str, _, resource_str = clause.partition(":")
    actions = action_str.split("+") if action_str else []
    resource_name, _, args_str = resource_str.partition("/")
    resource_args = args_str.split(",") if args_str else []
    return {
        "type": "permission",
        "actions": actions,
        "resource_name": resource_name,
        "resource_args": resource_args,
    }


def parse_sub_role(clause: str) -> t.Dict[str, t.Any]:
    sub_role, sep, sub_role_args_str = clause.partition("/")
    sub_role_args = sub_role_args_str.split(",") if sub_role_args_str else []
    return {
        "type": "role",
        "sub_role_name": sub_role,
        "sub_role_args": sub_role_args,
    }



import typing as t

from taktile_auth.entities import ResourceDefinition, RoleDefinition
from taktile_auth.parser.yaml_parsing import (
    parse_resource_yaml,
    parse_role_yaml,
)
from taktile_auth.settings import settings

RESOURCES: t.Dict[str, ResourceDefinition] = parse_resource_yaml(
    settings.RESOURCE_PATH
)
ROLES: t.Dict[str, RoleDefinition] = parse_role_yaml(
    settings.ROLE_PATH, settings.RESOURCE_PATH
)



import typing as t


class Cache(t.Protocol):
    """Cache Protocol
    Keep this in sync with the package defined in flow_services/packages/cache
    """

    def get(self, key: str) -> t.Optional[str]:
        """Get a value"""

    def put(
        self, key: str, value: str, time_to_live: t.Optional[int] = None
    ) -> None:
        """Put a value"""

    def delete(self, key: str) -> None:
        """Delete a values"""
